"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensure = void 0;
var predicates_1 = require("./predicates");
/**
 * @desc The `ensure` function verifies if the value meets the specified {Predicate}s.
 *
 * @example <caption>Basic usage</caption>
 * import { ensure, isDefined } from 'tiny-types'
 *
 * const username = 'jan-molak'
 * ensure('Username', username, isDefined());
 *
 * @example <caption>Ensuring validity of a domain object upon creation</caption>
 * import { TinyType, ensure, isDefined, isInteger, isInRange } from 'tiny-types'
 *
 * class Age extends TinyType {
 *   constructor(public readonly value: number) {
 *     ensure('Age', value, isDefined(), isInteger(), isInRange(0, 125));
 *   }
 * }
 *
 * @param {string} name - the name of the value to check.
 *      This name will be included in the error message should the check fail
 * @param {T} value - the argument to check
 * @param {...Array<Predicate<T>>} predicates - a list of predicates to check the value against
 * @returns {T} - if the original value passes all the predicates, it's returned from the function
 */
function ensure(name, value) {
    var predicates = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        predicates[_i - 2] = arguments[_i];
    }
    var result = predicates_1.and.apply(void 0, predicates).check(value);
    if (result instanceof predicates_1.Failure) {
        throw new Error("".concat(name, " should ").concat(result.description)); // eslint-disable-line unicorn/prefer-type-error
    }
    return result.value;
}
exports.ensure = ensure;
//# sourceMappingURL=ensure.js.map